#include "stack.h"
#include "../UART/UART.h"

/*NIC Einstellungen !! -----------------------------------
#########################################################*/

volatile unsigned char myip[4] = 	{192,168,1,147};
volatile unsigned char destip[4] = 	{192,168,1,120};
volatile u08 mymac[6] = 			{0x00,0x1C,0x25,0x00,0x60,0x61};
volatile unsigned char dummymac[6] = {0x00,0x00,0x00,0x00,0x00,0x00};

/*#######################################################
# Funktion um die Checksumme zu berechnen, stammt 		#
# ursprnglich von										#
# Ulrich Radig :-),www.ulrichradig.de 					#
# wer genaueres zur Checksummenbildung wissen mchte:	#
# http://www.netfor2.com/udpsum.htm 	 				#
########################################################*/
int checksum (unsigned char * pointer,unsigned long result32,unsigned int result16)
{
	unsigned int result16_1 = 0x0000;
	unsigned char DataH;
	unsigned char DataL;
	
	//Jetzt werden alle Packete in einer While Schleife addiert
	while(result16 > 1)
		{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH=*pointer++;

		//schreibt Inhalt Pointer nach DATAL danach inc Pointer
		DataL=*pointer++;

		//erzeugt Int aus Data L und Data H
		result16_1 = ((DataH << 8)+DataL);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
		//decrimiert Lnge von TCP Headerschleife um 2
		result16 -=2;
		}

	//Ist der Wert result16 ungerade ist DataL = 0
	if(result16 > 0)
		{
		//schreibt Inhalt Pointer nach DATAH danach inc Pointer
		DataH=*pointer;
		//erzeugt Int aus Data L ist 0 (ist nicht in der Berechnung) und Data H
		result16_1 = (DataH << 8);
		//Addiert packet mit vorherigen
		result32 = result32 + result16_1;
		}
	
	//Komplementbildung (addiert Long INT_H Byte mit Long INT L Byte)
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));
	result32 = ((result32 & 0x0000FFFF)+ ((result32 & 0xFFFF0000) >> 16));	
	result16 =~(result32 & 0x0000FFFF);
	
return (result16);
}
/*	Funktionen, um zu berprfen, ob das Packet fr uns bestimmt ist, weitere Prf'routinen'
	sind als Macros in stack.h definiert, da sie unabhngig von der IP immer auf dieselbe Art
	geprft werden knnen
######################################################################################*/
uint8_t Checkmymac(void){
	if(ETH_buffer[0] == mymac[0] && ETH_buffer[1] == mymac[1] && ETH_buffer[2] == mymac[2] && ETH_buffer[3] == mymac[3] && ETH_buffer[4] == mymac[4] && ETH_buffer[5] == mymac[5])return(1);
	return(0);
}

uint8_t Checkmyip(void){
	uart_send_var(ETH_buffer[38]);
	uart_putc('.');
	uart_send_var(ETH_buffer[39]);
	uart_putc('.');
	uart_send_var(ETH_buffer[40]);
	uart_putc('.');
	uart_send_var(ETH_buffer[41]);
	uart_putc('\n');

	if(ETH_buffer[38] == myip[0] && ETH_buffer[39] == myip[1] && ETH_buffer[40] == myip[2] && ETH_buffer[41] == myip[3])return(1);
	return(0);
}

/*Funktion um den Ethernet II Header zu erzeugen
####################################################################################*/
void eth(unsigned char *buff){
	
	for(unsigned char a=0; a<6; a++)
		{		
			buff[a] = buff[a+6];
			buff[a+6] = mymac[a];
		}
}
/*Funktion um den IP Header zu erzeugen
####################################################################################*/
void ip(unsigned char *buff){
	struct IP_header *ip;
	unsigned int sum;
	
	ip = (struct IP_Header*)&buff[24];

	for(unsigned char a=0;a<4; a++)
	{		
		ip->IP_destIp[a] = ip->IP_sourceIp[a];
		ip->IP_sourceIp[a] = myip[a];
	}


	ip->IP_checksum = 0x00; // checksum auf null setzen
	sum = checksum(&buff[14],0x00000000,20); // checksumme ausrechnen
	
	ip->IP_checksum = ((sum & 0xFF00) >> 8)|((sum & 0x00FF)<<8); 
}
/*Funktion um auf ein Arp Request eine Antwort zurck zu senden
####################################################################################*/
void arp(unsigned int len, unsigned char *buff){
		
	eth(buff); // ETHERNET II header erzeugen
	struct ARP_header *arp;
	arp = (struct ARP_Header *)&buff[21];
	arp->Opcode = 0x02; //Reply einstellen
	for(unsigned char a=0; a<6; a++)
		{			
			arp->ARP_destMac[a] = arp->ARP_sourceMac[a];
			arp->ARP_sourceMac[a] = mymac[a];
		}
	for(unsigned char a=0; a<4; a++)
		{		
			arp->ARP_destIp[a] = 	arp->ARP_sourceIp[a];
			arp->ARP_sourceIp[a] = 	myip[a];
		}
	
	
	enc28j60PacketSend(len-4,buff);	
}
/*Funktion um ICMP packete zu beantworten
####################################################################################*/
void icmp(unsigned int len, unsigned char *buff){
	eth(buff); // ETHERNET II header erzeugen
	ip(buff); // IP header erzeugen
	
	struct ICMP_header *icmp;
	icmp = (struct ICMP_Header *)&buff[34];
	
	icmp->ICMP_type = 0x00; // auf reply einstellen
	icmp->ICMP_code = 0x00; 
	
	//Simple ICMP Checksummenbildung, die Idee stammt von
	//Simon, siehe http://avr.auctionant.de/
	if(icmp->ICMP_checksumByteOne >  0xFF-0x08)icmp->ICMP_checksumByteTwo++;
	icmp->ICMP_checksumByteOne+=0x08; 
	
	enc28j60PacketSend(len-4,buff);	
}

/*UDP Funktion, beliebige Antwort
####################################################################################*/
void udp(unsigned int len, unsigned char *buff){ 
	uint16_t tempport,sum;
	struct UDP_header *udp;
	udp = (struct UDP_Header *)&buff[34];
	
	// ETHERNET II header erzeugen
	eth(buff); 
	
	// IP header erzeugen
	buff[16] = ((IP_UDP_HEADERLENGHT+len) & 0xFF00)>>8;
	buff[17] = ((IP_UDP_HEADERLENGHT+len) & 0x00FF); // ip header lenght anpassen
	
	ip(buff); 
		
	//An den Port zurcksenden, von dem das Packet gekommen ist
	tempport = udp->UDP_destPort; // Ziel Port zwischenspeichern
	udp->UDP_destPort = udp->UDP_sourcePort; //ZielPort neuschreiben
	udp->UDP_sourcePort = tempport; // SourcePort neuschreiben

	//Udp Header&Datenlnge schreiben
	udp->UDP_lenght_h = ((UDP_HEADERLENGHT+len) & 0xFF00)>>8;
	udp->UDP_lenght_l = ((UDP_HEADERLENGHT+len) & 0x00FF);
	
	//Checksumme ausrechnen...
	udp->UDP_checksum = 0x00;
	sum = checksum(&buff[26],TYPE_UDP+UDP_HEADERLENGHT+len,16+len); 
	udp->UDP_checksum = ((sum & 0xFF00) >> 8)|((sum & 0x00FF)<<8);
	
	//Und ab damit :-)
	enc28j60PacketSend(IP_UDP_HEADERLENGHT + ETH_HEADERLENGHT +len,buff);	
}


///////////////////////////////Eigene Funktionen//////////////////////////////7

/*Funktion um einen Individuellen IP Header zu erzeugen
####################################################################################*/
void ip_ind(unsigned char *buff, unsigned int len){
	struct IP_header_complete *ip;
	unsigned int sum;

	ip = (struct IP_header_complete *)&buff[14];

	for(unsigned char a=0;a<4; a++)
	{
		ip->IP_destIp[a] = destip[a];
		ip->IP_sourceIp[a] = myip[a];
	}

	ip->IP_version_header_Length = 0x45; //IPv4 und 20bytes Header
	ip->IP_serviceFields = 0x00;

	// IP header erzeugen
	buff[16] = ((IP_UDP_HEADERLENGHT+len) & 0xFF00)>>8;
	buff[17] = ((IP_UDP_HEADERLENGHT+len) & 0x00FF); // ip header lenght anpassen

	ip->IP_identification = 0x0000;
	ip->IP_Flag_Fragment_Offset = 0x0040;	//Bytes getauscht!
	ip->IP_TTL = 64;	//Time to Live wert der Paketes
	ip->IP_protocol = TYPE_UDP;	//UDP Paket

	// Checksumme generieren und in Header schreiben
	ip->IP_checksum = 0x00; // checksum auf null setzen
	sum = checksum(&buff[14],0x00000000,20); // checksumme ausrechnen
	ip->IP_checksum = ((sum & 0xFF00) >> 8)|((sum & 0x00FF)<<8);
}

/*Funktion um den Ethernet II Header zu erzeugen
####################################################################################*/
void eth_ind(unsigned char *buff){

	for(unsigned char a=0; a<6; a++)
		{
			buff[a] = dummymac[a];	//Ziel mac setzen
			buff[a+6] = mymac[a];	//Quell mac setzen
		}
	buff[12] = 0x08;	//Internet IPv4
	buff[13] = 0x00;
}

/*UDP Funktion, beliebige Antwort
####################################################################################*/
void send_over_udp(unsigned int len, unsigned char *buff, unsigned int uiDestPort, unsigned int uiSrcPort){
	uint16_t sum;
	unsigned char ucTemp = 0;
	struct UDP_header *udp;
	udp = (struct UDP_header *)&buff[34];

	// ETHERNET header erzeugen
	eth_ind(buff);

	ip_ind(buff,len); //IP Header erzeugen

	// Ziel Port umwandeln
	ucTemp = (uiDestPort & 0xFF00) >> 8;
	uiDestPort = (uiDestPort & 0x00FF) << 8;
	uiDestPort = (uiDestPort & 0xFF00) | ucTemp;

	// Quell Port umwandeln
	ucTemp = (uiSrcPort & 0xFF00) >> 8;
	uiSrcPort = (uiSrcPort & 0x00FF) << 8;
	uiSrcPort = (uiSrcPort & 0xFF00) | ucTemp;

	//Ports definieren
	udp->UDP_destPort =  uiDestPort;	//ZielPort
	udp->UDP_sourcePort = uiSrcPort; 	//SourcePort

	//Udp Header&Datenlnge schreiben
	udp->UDP_lenght_h = ((UDP_HEADERLENGHT+len) & 0xFF00)>>8;
	udp->UDP_lenght_l = ((UDP_HEADERLENGHT+len) & 0x00FF);

	//Checksumme ausrechnen...
	udp->UDP_checksum = 0x00;
	sum = checksum(&buff[26],TYPE_UDP+UDP_HEADERLENGHT+len,16+len);
	udp->UDP_checksum = ((sum & 0xFF00) >> 8)|((sum & 0x00FF)<<8);

	//Und ab damit :-)
	enc28j60PacketSend(IP_UDP_HEADERLENGHT + ETH_HEADERLENGHT +len,buff);
}



